@@ -290,52 +290,9 @@ download <- function(url, ...) {
   }
 }
 
-knownContentTypes <- Map$new()
-knownContentTypes$mset(
-  html='text/html; charset=UTF-8',
-  htm='text/html; charset=UTF-8',
-  js='text/javascript',
-  css='text/css',
-  png='image/png',
-  jpg='image/jpeg',
-  jpeg='image/jpeg',
-  gif='image/gif',
-  svg='image/svg+xml',
-  txt='text/plain',
-  pdf='application/pdf',
-  ps='application/postscript',
-  xml='application/xml',
-  m3u='audio/x-mpegurl',
-  m4a='audio/mp4a-latm',
-  m4b='audio/mp4a-latm',
-  m4p='audio/mp4a-latm',
-  mp3='audio/mpeg',
-  wav='audio/x-wav',
-  m4u='video/vnd.mpegurl',
-  m4v='video/x-m4v',
-  mp4='video/mp4',
-  mpeg='video/mpeg',
-  mpg='video/mpeg',
-  avi='video/x-msvideo',
-  mov='video/quicktime',
-  ogg='application/ogg',
-  swf='application/x-shockwave-flash',
-  doc='application/msword',
-  xls='application/vnd.ms-excel',
-  ppt='application/vnd.ms-powerpoint',
-  xlsx='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
-  xltx='application/vnd.openxmlformats-officedocument.spreadsheetml.template',
-  potx='application/vnd.openxmlformats-officedocument.presentationml.template',
-  ppsx='application/vnd.openxmlformats-officedocument.presentationml.slideshow',
-  pptx='application/vnd.openxmlformats-officedocument.presentationml.presentation',
-  sldx='application/vnd.openxmlformats-officedocument.presentationml.slide',
-  docx='application/vnd.openxmlformats-officedocument.wordprocessingml.document',
-  dotx='application/vnd.openxmlformats-officedocument.wordprocessingml.template',
-  xlam='application/vnd.ms-excel.addin.macroEnabled.12',
-  xlsb='application/vnd.ms-excel.sheet.binary.macroEnabled.12')
-
-getContentType <- function(ext, defaultType='application/octet-stream') {
-  knownContentTypes$get(tolower(ext)) %OR% defaultType
+getContentType <- function(file, defaultType = 'application/octet-stream') {
+  subtype <- ifelse(grepl('[.]html?$', file), 'charset=UTF-8', '')
+  mime::guess_type(file, unknown = defaultType, subtype = subtype)
 }
 
 # Create a zero-arg function from a quoted expression and environment
@@ -705,11 +662,6 @@ dataTablesJSON <- function(data, req) {
       for (j in seq_len(ncol(fdata))[k]) fdata[, j] <- htmlEscape(fdata[, j])
     }
   }
-  # WAT: toJSON(list(x = matrix(nrow = 0, ncol = 1))) => {"x": } (#299)
-  if (nrow(fdata) == 0) fdata <- list()
-  # WAT: toJSON(list(x = matrix(1:2))) => {x: [ [1], [2] ]}, however,
-  # toJSON(list(x = matrix(1))) => {x: [ 1 ]} (loss of dimension, #429)
-  if (length(fdata) && all(dim(fdata) == 1)) fdata <- list(list(fdata[1, 1]))
 
   res <- toJSON(list(
     draw = as.integer(q$draw),
