@@ -42,6 +42,7 @@ sd_section("UI Inputs",
     "sliderInput",
     "submitButton",
     "textInput",
+    "passwordInput",
     "updateCheckboxGroupInput",
     "updateCheckboxInput",
     "updateDateInput",
@@ -107,6 +108,7 @@ sd_section("Reactive constructs",
     "isolate",
     "makeReactiveBinding",
     "observe",
+    "observeEvent",
     "reactive",
     "reactiveFileReader",
     "reactivePoll",
@@ -154,7 +156,8 @@ sd_section("Utility functions",
     "plotPNG",
     "repeatable",
     "shinyDeprecated",
-    "serverInfo"
+    "serverInfo",
+    "shiny-options"
   )
 )
 sd_section("Embedding",
